import { __assign } from "tslib";
import Ajv from 'ajv';
import ajvKeywords from 'ajv-keywords';
import cloneDeep from 'lodash/cloneDeep';
import defaults from 'lodash/defaults';
import every from 'lodash/every';
import findKey from 'lodash/findKey';
import includes from 'lodash/includes';
import map from 'lodash/map';
import pickBy from 'lodash/pickBy';
import reduce from 'lodash/reduce';
import startsWith from 'lodash/startsWith';
import trimStart from 'lodash/trimStart';
import without from 'lodash/without';
import { randomString } from '../../utils';
import { getDataModel } from '../DataTypes';
var ajv = new Ajv();
ajvKeywords(ajv, ['regexp', 'formatMaximum', 'formatMinimum']);
export var FULL_TEXT_SLUG = 'full_text_search';
var DEFAULT_DELIMITER = '___';
export function filter(filters, collection) {
    ajv.removeSchema(/^.*$/);
    var validators = Array.isArray(filters)
        ? filters.map(function (s) { return ajv.compile(s); })
        : [ajv.compile(filters)];
    if (Array.isArray(collection)) {
        return collection.filter(function (m) { return every(validators, function (v) { return v(m); }); });
    }
    return pickBy(collection, function (m) { return every(validators, function (v) { return v(m); }); });
}
export var createFullTextSearchFilter = function (schema, term) {
    var items = !!schema.properties
        ? Object.entries(schema.properties).reduce(function (itemsAccumulator, entry) {
            var _a, _b;
            var key = entry[0], item = entry[1];
            var model = getDataModel(item);
            var defaultItem = {
                key: key,
                schema: item,
                model: model,
            };
            if ((_a = item.type) === null || _a === void 0 ? void 0 : _a.includes('boolean')) {
                return itemsAccumulator;
            }
            if (!!model && (!item.type || !item.type.includes('object'))) {
                itemsAccumulator.push(__assign(__assign({}, defaultItem), { operator: (model === null || model === void 0 ? void 0 : model.operators) && 'contains' in model.operators
                        ? 'contains'
                        : 'is' }));
            }
            else if (!!model && ((_b = item.type) === null || _b === void 0 ? void 0 : _b.includes('object'))) {
                itemsAccumulator.push(__assign(__assign({}, defaultItem), { operator: 'key_contains', searchKey: findKey(item.properties, { description: 'key' }) }));
                itemsAccumulator.push(__assign(__assign({}, defaultItem), { operator: 'value_contains', searchKey: findKey(item.properties, { description: 'value' }) }));
            }
            return itemsAccumulator;
        }, [])
        : [];
    var filter = generateFullTextSearchAjvFilter(term, items);
    return filter;
};
var generateFullTextSearchAjvFilter = function (term, items) {
    return {
        $id: randomString(),
        title: FULL_TEXT_SLUG,
        anyOf: [
            {
                title: FULL_TEXT_SLUG,
                description: "Any field contains " + term,
                anyOf: items.map(function (_a) {
                    var _b;
                    var key = _a.key, schema = _a.schema, operator = _a.operator, model = _a.model, searchKey = _a.searchKey;
                    var val = searchKey ? (_b = {}, _b[searchKey] = term, _b) : term;
                    return model === null || model === void 0 ? void 0 : model.createFilter(key, operator, val, schema);
                }),
            },
        ],
    };
};
export var insertFullTextSearch = function (schema, filters, term) {
    if (filters === void 0) { filters = []; }
    var searchFilter = createFullTextSearchFilter(schema, term);
    return filters.concat(searchFilter);
};
export var removeFullTextSearch = function (filters) {
    return filters.filter(function (f) { return f.title !== FULL_TEXT_SLUG; });
};
export var createFilter = function (schema, signatures) {
    var anyOf = signatures.map(function (_a) {
        var field = _a.field, operator = _a.operator, value = _a.value;
        var subSchema = schema.properties[field];
        var model = getDataModel(subSchema);
        if (!model || !model.operators.hasOwnProperty(operator)) {
            return {};
        }
        return model.createFilter(field, operator, value, subSchema);
    });
    return {
        $id: randomString(),
        anyOf: anyOf,
    };
};
export var decodeFilter = function (schema, filter) {
    var signatures = filter.anyOf.map(function (f) {
        if (!f) {
            return null;
        }
        if (!f.properties &&
            (!f.anyOf || !f.anyOf.length || !f.anyOf[0].properties)) {
            return null;
        }
        var schemaField = f.anyOf
            ? Object.keys(f.anyOf[0].properties).shift()
            : Object.keys(f.properties).shift();
        var subSchema = schema.properties[schemaField];
        var model = getDataModel(subSchema);
        if (!model) {
            return {};
        }
        return model.decodeFilter(f);
    });
    return signatures.filter(function (s) { return s !== null; });
};
export var getOperators = function (schema, field) {
    var subSchema = schema.properties[field];
    var model = getDataModel(subSchema);
    if (!model) {
        return [];
    }
    return map(model.operators, function (value, key) { return ({
        slug: key,
        label: value.getLabel(subSchema),
    }); });
};
var flattenAccumulator = function (schema, delimiter, parentKey, accumulator) {
    if (parentKey === void 0) { parentKey = ''; }
    if (!accumulator) {
        accumulator = cloneDeep(schema);
        if (schema.properties) {
            accumulator.properties = {};
        }
    }
    if (schema.anyOf && Array.isArray(schema.anyOf)) {
        accumulator.anyOf = schema.anyOf.map(function (subSchema) {
            return flattenAccumulator(subSchema, delimiter, parentKey);
        });
    }
    return reduce(schema.properties, function (result, value, key) {
        var _a;
        if (typeof value === 'boolean') {
            return accumulator;
        }
        var newKey = parentKey + delimiter + key;
        (_a = value.required) === null || _a === void 0 ? void 0 : _a.forEach(function (requiredKey) {
            if (!result.required) {
                result.required = [];
            }
            else {
                result.required = without(result.required, key, parentKey + delimiter + key);
            }
            result.required.push(newKey + delimiter + requiredKey);
        });
        if (value.type !== 'object' ||
            (findKey(value.properties, { description: 'key' }) &&
                findKey(value.properties, { description: 'value' }))) {
            if (parentKey) {
                result.properties[newKey] = defaults(value, { title: key });
                return result;
            }
            result.properties[key] = value;
            return result;
        }
        return flattenAccumulator(value, delimiter, newKey, result);
    }, accumulator);
};
export var flattenSchema = function (schema, delimiter) {
    if (delimiter === void 0) { delimiter = DEFAULT_DELIMITER; }
    return flattenAccumulator(schema, delimiter);
};
export var unflattenSchema = function (schema, delimiter) {
    if (delimiter === void 0) { delimiter = DEFAULT_DELIMITER; }
    var base = cloneDeep(schema);
    if (schema.properties) {
        base.properties = {};
    }
    var unflattenedSchema = reduce(schema.properties, function (result, value, field) {
        if (!startsWith(field, delimiter)) {
            result.properties[field] = value;
            return result;
        }
        var keys = trimStart(field, delimiter).split(delimiter);
        var lastKey = keys.pop();
        var head = result;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            if (!head.properties[key]) {
                head.properties[key] = {
                    type: 'object',
                    properties: {},
                };
            }
            head = head.properties[key];
        }
        head.properties[lastKey] = value;
        return result;
    }, base);
    if (unflattenedSchema.anyOf) {
        unflattenedSchema.anyOf = unflattenedSchema.anyOf.map(function (subSchema) {
            return unflattenSchema(subSchema, delimiter);
        });
    }
    if (unflattenedSchema.required) {
        var requiredFields = unflattenedSchema.required;
        unflattenedSchema.required = [];
        requiredFields.forEach(function (requiredField) {
            var fields = trimStart(requiredField, delimiter).split(delimiter);
            var head = unflattenedSchema;
            for (var _i = 0, fields_1 = fields; _i < fields_1.length; _i++) {
                var field = fields_1[_i];
                if (!head.required) {
                    head.required = [];
                }
                if (!includes(head.required, field)) {
                    head.required.push(field);
                }
                head = head.properties[field];
            }
        });
    }
    return unflattenedSchema;
};
export var getCleanEditModel = function (schema, field) {
    if (!field) {
        field = Object.keys(schema.properties).shift();
    }
    var fieldOperators = getOperators(schema, field);
    if (!fieldOperators.length) {
        return {
            field: field,
            operator: '',
            value: '',
        };
    }
    var operator = fieldOperators.shift().slug;
    var value = '';
    var subschema = schema.properties[field];
    if (typeof subschema !== 'boolean') {
        if (subschema.enum) {
            value = subschema.enum[0] || '';
        }
        if (subschema.oneOf) {
            value = subschema.oneOf[0].const || '';
        }
        if (subschema.type === 'boolean') {
            value = true;
        }
    }
    return {
        field: field,
        operator: operator,
        value: value,
    };
};
