import { __assign } from "tslib";
import { mount } from 'enzyme';
import React from 'react';
import { Filters, Provider, SchemaSieve, Select, Box } from '../../';
import FiltersSummary from '../../components/Filters/Summary';
import { Keyboard } from 'grommet';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
var schema = {
    type: 'object',
    properties: {
        Name: {
            title: 'Pokemon Name',
            type: 'string',
        },
        Tag: {
            type: 'object',
            properties: {
                tag_name: {
                    title: 'Name',
                    description: 'key',
                    type: 'string',
                },
                tag_value: {
                    description: 'value',
                    title: 'Value',
                    type: 'string',
                },
            },
        },
    },
};
var filter = SchemaSieve.createFilter(schema, [
    {
        field: 'Name',
        operator: 'is',
        value: 'Squirtle',
    },
]);
var tagIsFilter = SchemaSieve.createFilter(schema, [
    {
        field: 'Tag',
        operator: 'is',
        value: { tag_name: 'rarity', tag_value: '10' },
    },
]);
var tagIsNotFilter = SchemaSieve.createFilter(schema, [
    {
        field: 'Tag',
        operator: 'is_not',
        value: { tag_name: 'rarity', tag_value: '10' },
    },
]);
var view = {
    id: 'abcde',
    name: 'Test',
    scope: null,
    filters: [filter],
};
var viewScopes = [
    {
        slug: 'foo',
        name: 'foo',
    },
    {
        slug: 'bar',
        name: 'bar',
    },
];
describe('Filters component', function () {
    describe('filters property', function () {
        it('should not render a summary if there are no filters', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema })));
            expect(component.find(FiltersSummary)).toHaveLength(0);
            component.unmount();
        });
        it('should render a summary of filters that are provided as props', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter] })));
            expect(component.find(FiltersSummary)).toHaveLength(1);
            component.unmount();
        });
        it('should render a summary of filters that are added as props after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema }));
            component.setProps({ filters: [filter] });
            component.update();
            expect(component.find(FiltersSummary)).toHaveLength(1);
            component.unmount();
        });
        it('should not render a summary of filters if they are removed after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema, filters: [filter] }));
            component.setProps({ filters: null });
            component.update();
            expect(component.find(FiltersSummary)).toHaveLength(0);
            component.unmount();
        });
        it('should show the correct label when filtering using "is" operator for both key and value properties of an object', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [tagIsFilter] })));
            expect(component.find('button').at(4).text()).toBe('Tag is rarity : 10');
        });
        it('should show the correct label when filtering using "is_not" operator for both key and value properties of an object', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [tagIsNotFilter] })));
            expect(component.find('button').at(4).text()).toBe('Tag is not rarity : 10');
        });
        it('should remove buttons labels if compact property is set to true', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { compact: [true, true, true, true], schema: schema })));
            expect(component.find('button').at(0).text()).toBe('');
            expect(component.find('button').at(1).text()).toBe('');
        });
    });
    describe('save views modal', function () {
        it('should not show scopes selector if one or less scopes are passed', function () {
            var withOneViewScope = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter], viewScopes: [viewScopes[0]] })));
            var withNoViewScopes = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter], viewScopes: [viewScopes[0]] })));
            withOneViewScope.find('button').at(3).simulate('click');
            expect(withOneViewScope.find(Select)).toHaveLength(0);
            withNoViewScopes.find('button').at(3).simulate('click');
            expect(withNoViewScopes.find(Select)).toHaveLength(0);
            withOneViewScope.unmount();
            withNoViewScopes.unmount();
        });
        it('should show scopes selector if viewScopes are passed', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter], viewScopes: viewScopes })));
            component.find('button').at(3).simulate('click');
            expect(component.find(Select)).toHaveLength(1);
            component.unmount();
        });
    });
    describe('views property', function () {
        it('should not render a views menu if there are no views', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema })));
            component.find('button').at(1).simulate('click');
            expect(component.find(Box).at(1).html()).toContain("You haven't created any views yet");
            component.unmount();
        });
        it('should render a list of views that are provided as props', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [view] })));
            component.find('button').at(1).simulate('click');
            expect(component.find(FontAwesomeIcon)).toHaveLength(5);
            component.unmount();
        });
        it('should not render scoped views in the views menu if no scopes are passed', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [__assign(__assign({}, view), { scope: 'foo' })] })));
            component.find('button').at(1).simulate('click');
            expect(component.find("[children='foo']")).toHaveLength(0);
            expect(component.find("[children='null']")).toHaveLength(0);
            component.unmount();
        });
        it('should not render scoped views in the views menu if one or less scopes are passed', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [__assign(__assign({}, view), { scope: 'foo' })], viewScopes: [viewScopes[0]] })));
            component.find('button').at(1).simulate('click');
            expect(component.find("[children='foo']")).toHaveLength(0);
            component.unmount();
        });
        it('should render scoped views in the views menu if more than one viewScopes are passed', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [__assign(__assign({}, view), { scope: 'foo' })], viewScopes: viewScopes })));
            component.find('button').at(1).simulate('click');
            expect(component.find('strong').at(0).text()).toEqual('foo');
            expect(component.find("[children='bar']")).toHaveLength(0);
            component.unmount();
        });
        it('should render views as "Unscoped" if they do not have a scope and more than one viewScopes are passed', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [__assign({}, view)], viewScopes: viewScopes })));
            component.find('button').at(1).simulate('click');
            expect(component.find('strong').at(0).text()).toEqual('Unscoped');
            expect(component.find("[children='foo']")).toHaveLength(0);
            expect(component.find("[children='bar']")).toHaveLength(0);
            component.unmount();
        });
        it('should render a list of views that are added as props after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema }));
            component.setProps({ views: [view] });
            component.update();
            component.find('button').at(1).simulate('click');
            expect(component.find(FontAwesomeIcon)).toHaveLength(5);
            component.unmount();
        });
        it('should not render a list of views if they are removed after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema, views: [view] }));
            component.setProps({ views: null });
            component.update();
            component.find('button').at(1).simulate('click');
            expect(component.find(Box).at(1).html()).toContain("You haven't created any views yet");
            component.unmount();
        });
        it('should render when the schema contains an unknown type', function () {
            var unknownSchema = {
                type: 'object',
                properties: {
                    test: {
                        type: 'Foo Bar',
                    },
                },
            };
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: unknownSchema })));
            component.unmount();
        });
        it('should clear all filters when `clear all filters` gets clicked', function () {
            var defaultFilters = SchemaSieve.createFilter(schema, [
                { field: 'Name', operator: 'contains', value: 's' },
                { field: 'Name', operator: 'contains', value: 'q' },
            ]);
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [defaultFilters] })));
            expect(component.find(FiltersSummary)).toHaveLength(1);
            component
                .find('input')
                .simulate('change', { target: { value: 'Squirtle' } });
            component.find(Keyboard).simulate('keypress', { key: 'Enter' });
            expect(component.text()).toContain('Clear all');
            component
                .findWhere(function (node) { return node.text() === 'Clear all'; })
                .at(1)
                .simulate('click');
            expect(component.find(FiltersSummary)).toHaveLength(0);
            component.unmount();
        });
    });
});
